"""local api client used to obtain
IoT service credentials for iot mqtt communication

"""

import os
import logging
import requests

AppConfig = {
    "appId": "",
    "iotEndpoint": "",
    "thingName": "",
    "accountId": "",
    "key": "",
    "cert": "",
    "ca": ""
}

CONFIG_URL = "http://localhost:{}/api/thing"

def get_local_config(port=5000):
    """get thing configuration from local HTTP server
    """
    env_port = os.getenv('PORT')
    if env_port:
        port = env_port
    url = CONFIG_URL.format(port)
    return get_global_config(url)


def get_global_config(url):
    """get thing configuration from url
    """
    try:
        result = requests.get(url, timeout=300)
        result.raise_for_status()
        if result.status_code == 200:
            data = result.json()
            return data
    except requests.HTTPError:
        logging.error("signalsdk: Configuration can not be obtained")
    except AttributeError:
        logging.error("signalsdk: Invalid data from local thing api server.")
    return None



"""signalsdk
signalsdk module provide interface for signal applications to interact with
Teldio Signal platform.

Initialize SDK: Entry point for Signal Application SDK
* Creates a bi-directional communication between device shadow and application

initialize(applicationType: string,
     onConfigurationChangeRequestedCallback: OnConfigurationChangeRequested,
     onEventReceivedCallback: OnEventReceived)

Pass event to the next entity by sending an event to a designated topic
next(event: string)

signal application send report when configuration change is applied
reportConfigurationChange(event: object)

callback method used to handle configuration change when requested from the cloud
* Triggered when a new configuration change requested from the cloud
onConfigurationChangeRequested(newConfig: object)

callback method used to handle the events received from event bus
* Triggered when a new event received from application's subscribed topic
onEventReceived(event: object)

"""
import sys
import os
import logging

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))
print("package initialized.")
logging.basicConfig(format='%(asctime)s - %(message)s', level=logging.INFO)



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

import os

from awscrt.io import (
    ClientBootstrap,
    DefaultHostResolver,
    EventLoopGroup,
    SocketDomain,
    SocketOptions,
)
from awsiot.eventstreamrpc import (
    Connection,
    LifecycleHandler,
    MessageAmendment
)

TIMEOUT = 10


class IpcUtils:
    def __init__(self):
        self.lifecycle_handler = LifecycleHandler()

    def connect(self):
        elg = EventLoopGroup()
        resolver = DefaultHostResolver(elg)
        bootstrap = ClientBootstrap(elg, resolver)
        socket_options = SocketOptions()
        socket_options.domain = SocketDomain.Local
        amender = MessageAmendment.create_static_authtoken_amender(os.getenv("SVCUID"))
        hostname = os.getenv("AWS_GG_NUCLEUS_DOMAIN_SOCKET_FILEPATH_FOR_COMPONENT")
        connection = Connection(
            host_name=hostname,
            port=8033,
            bootstrap=bootstrap,
            socket_options=socket_options,
            connect_message_amender=amender,
        )

        connect_future = connection.connect(self.lifecycle_handler)
        connect_future.result(TIMEOUT)
        return connection



import os
import logging
from signalsdk.shadow_client import OnConfigurationChangeRequested, ShadowClient
from signalsdk.iot_mqtt_connection import IotMqttConnection
from signalsdk.local_mqtt import OnEventReceived, LocalMqtt
from signalsdk.api import get_local_config
from signalsdk.application_config import get_config_for_app
from signalsdk.config import LOCALMQTT_SDK_TOPIC_PREFIX
from signalsdk.signal_exception import SignalAppCallBackError, SignalAppConfigEnvError, \
    SignalAppInitError, \
    SignalDeploymentNotFoundError
from .validator import throw_if_parameter_not_found_in

class SignalApp:
    def __init__(self):
        self.iotMqttConnection = None
        self.shadow_client = None
        self.localMqtt = None
        self.appConfig = {}
        self.appSettings = {}
        self.app_id = ""
        self.thing_name = ""
        self.account_id = ""
        self.deviceDeploymentVersion = None
        self.deviceDeploymentId = None
        self.groupDeploymentVersion = None
        self.groupDeploymentId = None
        self.deviceApiDomainName = None
        self.configurationChangedCallback = None
        self.onEventReceivedCallback = None

    def on_config_change_requested(self, delta):
        logging.info(f"signalsdk: configuration change received: "
                     f"{str(delta)}")

        self.deviceApiDomainName = delta.get("deviceApiDomainName") or self.deviceApiDomainName

        throw_if_parameter_not_found_in(self.deviceApiDomainName,'device api','device shadow')

        self.deviceDeploymentVersion = delta.get("deploymentVersion")
        logging.info(f"signalsdk::device deploymentVersion: {self.deviceDeploymentVersion}")
        # if deployment version is null then it means
        # deployment does not exist at all/is deleted
        if not self.deviceDeploymentVersion:
            self.deviceDeploymentId = None
        else:
            self.deviceDeploymentId = delta.get("deploymentId") or self.deviceDeploymentId
            logging.info(f"signalsdk::device deploymentId: {self.deviceDeploymentId}")

        self.groupDeploymentVersion = delta.get("groupDeploymentVersion")
        logging.info(f"signalsdk::group deploymentVersion: {self.groupDeploymentVersion}")
        # if group deployment version is null then it means
        # group deployment does not exist at all/is deleted
        if not self.groupDeploymentVersion:
            self.groupDeploymentId = None
        else:
            self.groupDeploymentId = delta.get("groupDeploymentId") or self.groupDeploymentId
            logging.info(f"signalsdk::group deploymentId: {self.groupDeploymentId}")

        if not self.deviceDeploymentId and not self.groupDeploymentId:
            raise SignalDeploymentNotFoundError

        self.appSettings = get_config_for_app(self.deviceDeploymentId,
                                              self.groupDeploymentId,
                                              self.account_id,
                                              self.app_id,
                                              self.deviceApiDomainName)
        current_subtopic = self.localMqtt.get_subscribed_topic()
        if "sdkSubTopic" in self.appSettings["settingsForSDK"] and \
            self.appSettings["settingsForSDK"]["sdkSubTopic"]:
            desired_subtopic = LOCALMQTT_SDK_TOPIC_PREFIX \
                               + self.appSettings["settingsForSDK"]["sdkSubTopic"]
        else:
            desired_subtopic = LOCALMQTT_SDK_TOPIC_PREFIX + self.app_id
        self.renew_topic_subscription(current_subtopic, desired_subtopic)

        if self.appSettings["settingsForApp"] != {}:
            try:
                self.configurationChangedCallback(self.appSettings["settingsForApp"])
            except Exception as err:
                logging.info(f"signalsdk: configurationChangedCallback "
                             f"function threw an error: {err}")
                raise SignalAppCallBackError from err
        else:
            self.report_configuration_change()

    def initialize(self, onConfiguratioChangedCallback: OnConfigurationChangeRequested,
                   onEventReceivedCallback: OnEventReceived):
        """Signal Application Initialize
        Following objects are created
        localMqtt: it is used to subscribe or publish to local MQTT broker
        served as local event bus
        iotMqttConnection: it is used subscribe or publish device
        shadow configuration topics
        shadow_client: it listens to device shadow change topic and trigger
        local updates and reports

        :param application_type:
        :param onConfiguratioChangedCallback: call back function provided by
        signal application for configuration change
        :param onEventReceivedCallback: call back function provided by signal
        application for events handling
        """
        logging.info("signalsdk::Starting signal app initialize.")
        self.configurationChangedCallback = onConfiguratioChangedCallback
        self.onEventReceivedCallback = onEventReceivedCallback
        self.appConfig = get_local_config()
        self.thing_name = self.appConfig['thingName']
        self.account_id = self.appConfig['accountId']
        self.app_id = os.getenv('APPLICATION_ID')
        throw_if_parameter_not_found_in(self.app_id, 'application id', 'environment variables')
        self.shadow_client = None
        if not self.app_id:
            raise SignalAppConfigEnvError

        self.localMqtt = LocalMqtt()
        self.localMqtt.set_on_event_received(self.onEventReceivedCallback)
        self.localMqtt.connect()

        self.iotMqttConnection = IotMqttConnection(self.app_id, self.appConfig)
        self.iotMqttConnection.set_on_connection_successful(
            self.start_listening_configuration_updates)
        self.iotMqttConnection.connect()
        return self.iotMqttConnection

    def start_listening_configuration_updates(self):
        logging.info("signalsdk:IoT Connected. Starting to listen configuration updates...")
        if not self.shadow_client:
            self.shadow_client = ShadowClient(self.iotMqttConnection, self.thing_name)
            logging.info("signalsdk:Shadow client created")
        self.shadow_client.set_on_configuration_change_requested(self.on_config_change_requested)
        self.shadow_client.get_application_config()
        self.shadow_client.start_listening_device_shadow()

    def next(self, event, next_app_id=''):
        """Publish the event
        :param event:
        :return:
        """
        if 'settingsForSDK' not in self.appSettings and not next_app_id:
            logging.info(f"signalsdk:next called without topic to publish to: {self.appSettings}"
                         f"event: {event} ")
            return
        logging.info(f"signalsdk: forwarding event to topic: "
                     f"event: {event}")

        if not self.shadow_client:
            logging.error("signalsdk: shadow client is not created")
            raise SignalAppInitError

        if 'sdkPubTopic' in self.appSettings['settingsForSDK'] and \
                self.appSettings['settingsForSDK']['sdkPubTopic']:
            topic = LOCALMQTT_SDK_TOPIC_PREFIX + self.appSettings['settingsForSDK']['sdkPubTopic']
            logging.info(f"signalsdk next() publishing to sdk topic: "
                         f"{topic}")
            self.localMqtt.publish(topic, event)
        elif next_app_id:
            topic = LOCALMQTT_SDK_TOPIC_PREFIX + next_app_id
            logging.info(f"signalsdk next() publishing to applicationId topic: {topic}")
            self.localMqtt.publish(topic, event)

    def report_configuration_change(self):
        logging.info("signalsdk: report_configuration_change")
        if not self.shadow_client:
            logging.error("signalsdk: shadow client is not created")
            raise SignalAppInitError
        to_report = {}
        if self.deviceDeploymentId:
            to_report["deploymentId"] = self.deviceDeploymentId
        if self.deviceDeploymentVersion:
            to_report["deploymentVersion"] = self.deviceDeploymentVersion
        if self.groupDeploymentId:
            to_report["groupDeploymentId"] = self.groupDeploymentId
        if self.groupDeploymentVersion:
            to_report["groupDeploymentVersion"] = self.groupDeploymentVersion
        to_report["deviceApiDomainName"] = self.deviceApiDomainName
        logging.info(f"signalsdk: report configuration change with: {to_report}")
        self.shadow_client.update_application_config(to_report)

    def renew_topic_subscription(self, current_topic, desired_topic):
        logging.info(f"signalsdk:renew_topic_subscription current_topic: {current_topic} "
                     f"desired_topic: {desired_topic}")
        if current_topic and current_topic != desired_topic:
            self.localMqtt.unsubscribe()
        if desired_topic and current_topic != desired_topic:
            self.localMqtt.subscribe(desired_topic)



import logging
from typing import Callable
from awscrt.mqtt import QoS
from awsiot import iotshadow
from signalsdk.iot_mqtt_connection import IotMqttConnection

OnConfigurationChangeRequested=Callable[[object], None]

class ShadowClient:
    """IoT device shadow client
    it is responsible of communication with device shadow on configuration changes

    """
    def __init__(self, iot_mqtt_connection: IotMqttConnection, thingName):
        self.shadow_client = iotshadow.IotShadowClient(iot_mqtt_connection.get_connection())
        self.configuration_changed_callback = None
        self.shadowName = 'applications-shadow'
        self.thingName = thingName

    def set_on_configuration_change_requested(self, callback: OnConfigurationChangeRequested):
        self.configuration_changed_callback = callback

    def on_named_shadow_delta_updated(self, response: iotshadow.ShadowDeltaUpdatedEvent):
        logging.info(f"signalsdk::{__name__}:on_named_shadow_delta_updated. "
                     f"Triggering callback. Response: {response.state} "
                     f"Type:{type(response.state)}")
        self.configuration_changed_callback(response.state)

    def on_named_shadow_updated(self, response: iotshadow.ShadowUpdatedEvent):
        logging.info(f"signalsdk::{__name__}:on_named_shadow_updated. {response}")

    def on_update_named_shadow_accepted(self, response: iotshadow.UpdateShadowResponse):
        logging.info(f"signalsdk::{__name__}:on_update_named_shadow_accepted. "
                     f"Current config: {response}")

    def on_update_named_shadow_rejected(self, response: iotshadow.ErrorResponse):
        logging.info(f"{__name__}:on_update_named_shadow_rejected. "
                     f"Current config: {response}")

    def on_get_named_shadow_accepted(self, response: iotshadow.GetShadowResponse):
        logging.info(f"signalsdk::{__name__}:on_get_named_shadow_accepted. "
                     f"Triggering callback. Response: {response.state} "
                     f"Type:{type(response.state)}")
        delta = response.state.desired
        self.configuration_changed_callback(delta)

    def on_get_named_shadow_rejected(self, response: iotshadow.ErrorResponse):
        logging.warning(f"signalsdk::{__name__}:on_get_named_shadow_rejected. "
                     f"Response: {response}")

    def on_publish_update_named_shadow(self, future):
        # type: (Future) -> None
        try:
            future.result()
            logging.info("signalsdk::Update named shadow published.")
        except Exception as e:
            logging.error(f"signalsdk::Failed to publish update named shadow request. Error: {e}")

    def start_listening_device_shadow(self):
        logging.info(f"signalsdk::{__name__}:start_listening_device_shadow. "
                     f"thing_name:{self.thingName} shadow_name:{self.shadowName}")
        delta_updated_subscribed_future, _ = \
            self.shadow_client.subscribe_to_named_shadow_delta_updated_events(
            request=iotshadow.NamedShadowDeltaUpdatedSubscriptionRequest(
                thing_name=self.thingName, shadow_name=self.shadowName),
            qos=QoS.AT_LEAST_ONCE,
            callback=self.on_named_shadow_delta_updated
            )
        delta_updated_subscribed_future.result()

        shadow_updated_subscribed_future, _ = \
            self.shadow_client.subscribe_to_named_shadow_updated_events(
            request=iotshadow.NamedShadowUpdatedSubscriptionRequest(
                thing_name=self.thingName, shadow_name=self.shadowName),
            qos=QoS.AT_LEAST_ONCE,
            callback=self.on_named_shadow_updated
        )
        shadow_updated_subscribed_future.result()

        update_accepted_subscribed_future, _ = \
            self.shadow_client.subscribe_to_update_named_shadow_accepted(
            request=iotshadow.UpdateNamedShadowSubscriptionRequest(
                thing_name=self.thingName, shadow_name=self.shadowName),
            qos=QoS.AT_LEAST_ONCE,
            callback=self.on_update_named_shadow_accepted
        )
        update_accepted_subscribed_future.result()

        update_rejected_subscribed_future, _ = \
            self.shadow_client.subscribe_to_update_named_shadow_rejected(
            request=iotshadow.UpdateNamedShadowSubscriptionRequest(
                thing_name=self.thingName, shadow_name=self.shadowName),
            qos=QoS.AT_LEAST_ONCE,
            callback=self.on_update_named_shadow_rejected
        )
        update_rejected_subscribed_future.result()

    def get_application_config(self):
        logging.info(f"signalsdk::{__name__}:get_application_config. "
                     f"thing_name:{self.thingName} shadow_name:{self.shadowName}")
        get_accepted_subscribed_future, _ = \
            self.shadow_client.subscribe_to_get_named_shadow_accepted(
            request=iotshadow.GetNamedShadowSubscriptionRequest(
                thing_name=self.thingName, shadow_name=self.shadowName),
            qos=QoS.AT_LEAST_ONCE,
            callback=self.on_get_named_shadow_accepted
            )
        get_accepted_subscribed_future.result()
        logging.info("signalsdk::subscribed to get_named_shadow_accepted")

        get_rejected_subscribed_future, _ = \
            self.shadow_client.subscribe_to_get_named_shadow_rejected(
            request=iotshadow.GetNamedShadowSubscriptionRequest(
                thing_name=self.thingName, shadow_name=self.shadowName),
            qos=QoS.AT_LEAST_ONCE,
            callback=self.on_get_named_shadow_rejected
        )
        get_rejected_subscribed_future.result()
        logging.info("signalsdk::subscribed to get_named_shadow_rejected")

        publish_get_future = self.shadow_client.publish_get_named_shadow(
            request=iotshadow.GetNamedShadowRequest(thing_name=self.thingName,
                                                    shadow_name=self.shadowName),
            qos=QoS.AT_LEAST_ONCE
        )
        publish_get_future.result()
        logging.info("signalsdk::published get_named_shadow")

    def update_application_config(self, config):
        logging.info(f"signalsdk::{__name__}:update_application_config. "
                     f"thing_name:{self.thingName} shadow_name:{self.shadowName}")
        publish_update_future = self.shadow_client.publish_update_named_shadow(
            request=iotshadow.UpdateNamedShadowRequest(
                thing_name=self.thingName,
                shadow_name=self.shadowName,
                state=iotshadow.ShadowState(reported=config)),
            qos=QoS.AT_LEAST_ONCE
            )
        publish_update_future.add_done_callback(self.on_publish_update_named_shadow)



import logging
from dateutil import parser
from signalsdk.signed_request_client import signed_request
from signalsdk.signal_exception import SignalAppParamNotFoundError

Application = {
    "id": "",
    "name": "string",
    "localVersion": "",
    "versionOnGreengrass": "",
    "sdkPubTopic": "",
    "sdkSubTopic": "",
    "settings": []
}

DeploymentInfo = {
    "applications": [],
    "name": "",
    "createdAt": "",
    "deploymentVersion": ""
}

ApplicationConfig = {
    "version": "",
    "createdAt": "",
    "settingsForApp": {},
    "settingsForSDK": {}
}


def get_config_for_app(device_deployment_id,
                       group_deployment_id,
                       account_id,
                       application_id,
                       device_api_dn):
    if not device_deployment_id and not group_deployment_id:
        logging.error("Neither device deployment id nor group deployment id exists")
        raise SignalAppParamNotFoundError("deploymentId and groupDeploymentId")
    app_device_deployment_settings = None
    app_group_deployment_settings = None
    if device_deployment_id:
        logging.info("signalsdk::Retrieving device deployment info...")
        app_device_deployment_settings = get_app_config_for_deployment(
            device_deployment_id, account_id, application_id, device_api_dn)
    if group_deployment_id:
        logging.info("signalsdk::Retrieving group deployment info...")
        app_group_deployment_settings = get_app_config_for_deployment(
            group_deployment_id, account_id, application_id, device_api_dn)
    if not app_device_deployment_settings:
        return app_group_deployment_settings
    if not app_group_deployment_settings:
        return app_device_deployment_settings
    device_deployment_date = parser.parse(app_device_deployment_settings["createdAt"])
    group_deployment_date = parser.parse(app_group_deployment_settings["createdAt"])
    if device_deployment_date > group_deployment_date:
        return app_device_deployment_settings
    return app_group_deployment_settings

def get_app_config_for_deployment(deployment_id, account_id, application_id, device_api_dn):
    api_path = "".join(["/accounts/", account_id, "/deployments/", deployment_id])
    api_url = format_base_url(device_api_dn + api_path)
    logging.info(f"signalsdk::sending signed request: url: {api_url}")
    deploymentInfo = signed_request(method='GET', url=api_url, data={}, headers={})
    logging.info(f"signalsdk::Deployment Info: {deploymentInfo}")
    app_list = deploymentInfo["applications"]
    final_config = {}
    for app in app_list:
        if app["id"] == application_id:
            applicationInfo = app
    if applicationInfo:
        app_settings = {}
        for item in applicationInfo["settings"]:
            app_settings[item['key']] = item['value']
        final_config = {"createdAt": deploymentInfo["createdAt"],
                        "version": applicationInfo["versionOnGreengrass"],
                        "settingsForSDK": {"sdkPubTopic": applicationInfo["sdkPubTopic"],
                                           "sdkSubTopic": applicationInfo["sdkSubTopic"]},
                        "settingsForApp": app_settings}
    return final_config

def format_base_url(base_url: str):
    if base_url.startswith("https://") or base_url.startswith("http://"):
        return base_url
    return "https://" + base_url



"""Mqtt Connection to AWS IoT service
 IoTMqttConnection class is used to create MQTT connection
 between signal application and aws IoT cloud service
"""

import logging
from typing import Callable
from awscrt.io import ClientBootstrap, EventLoopGroup, DefaultHostResolver
from awscrt.mqtt import QoS
from awscrt.exceptions import AwsCrtError
from awsiot import mqtt_connection_builder
from signalsdk.signal_exception import SignalAppIoTConError

OnConnectionSuccessful=Callable[[], None]

class IotMqttConnection:
    """
    A class used to represent Mqtt Connection to AWS IoT service
     IoTMqttConnection class is used to create MQTT connection
     between signal application and aws IoT cloud service
    """

    def __init__(self, app_id, app_config):
        """
        params:
        app_id application Id
        thing_config aws IoT thing configurations including:
        thing name, account Id, device certificate, private
        keys etc
        """

        self.is_connected = False
        self.thingName = app_config["thingName"]
        self.accountId = app_config["accountId"]
        self.appId = self.thingName + "-" + app_id
        self.on_connection_successful_callback = None

        event_loop_group = EventLoopGroup(1)
        client_bootstrap = ClientBootstrap(event_loop_group, DefaultHostResolver(event_loop_group))

        self.mqtt_connection = mqtt_connection_builder.mtls_from_bytes(
            endpoint=app_config["iotEndpoint"],
            cert_bytes=str.encode(app_config["cert"]),
            pri_key_bytes=str.encode(app_config["key"]),
            client_bootstrap=client_bootstrap,
            ca_bytes=str.encode(app_config["ca"]),
            client_id=self.appId,
            clean_session=False,
            keep_alive_secs=30,
            on_connection_interrupted=self.on_connection_interrupted,
            on_connection_resumed=self.on_connection_resumed
        )

    def on_connection_interrupted(self, connection, error, **kwargs):
        logging.warning(f"connection interrupted. error: {error}")

    def on_connection_resumed(self, connection, return_code, session_present, **kwargs):
        logging.warning("connection resumed.")

    def set_on_connection_successful(self, callback):
        self.on_connection_successful_callback  = callback

    def connect(self):
        """
        connect
        """
        logging.info("signalsdk::connecting IotMqtt...")
        if not self.is_connected:
            connected_future = self.mqtt_connection.connect()
            try:
                connected_future.result()
                self.is_connected = True
                logging.info("signalsdk::Connected!")
                self.on_connection_successful_callback()
            except AwsCrtError as err:
                raise SignalAppIoTConError from err

    def get_connection(self):
        return self.mqtt_connection

    def publish(self, topic, payload):
        logging.info(f"signalsdk: iotMqtt publish with: topic:{topic} payload:{payload}")
        publish_future, _ = self.mqtt_connection.publish(topic, payload, QoS.AT_LEAST_ONCE)
        publish_future.result()



SDK_INIT_ERROR = 'signalsdk::Please initialize Signal Application SDK first!'
SDK_APP_CONFIG_ENV_ERROR = 'signalsdk::Check deployment environment!'
SDK_APP_ID_NOT_FOUND_ERROR = 'signalsdk::Application Id not found on environment!'
SDK_APP_PARAM_NOT_FOUND_ERROR = "signalsdk::Parameter not found"
SDK_APP_CALL_BACK_ERROR = "signalsdk::config change cb function threw an error"
SDK_APP_IOT_CONN_ERROR = "signalsdk::IoT MQTT connection error"
SDK_APP_SIGNED_REQ_ERROR = "signalsdk:signed request error"
SDK_DEPLOYMENT_ID_NOT_FOUND_ERROR = ('signalsdk:deploymentId  or groupDeploymentId '
                                     'not found on config error')
SDK_DEVICE_DOMAIN_NOT_FOUND_ERROR = "signalsdk:device api domain not found on config error"


class SignalAppInitError(Exception):
    def __str__(self):
        return SDK_INIT_ERROR


class SignalAppConfigEnvError(Exception):
    def __str__(self):
        return SDK_APP_ID_NOT_FOUND_ERROR


class SignalDeploymentNotFoundError(Exception):
    def __str__(self):
        return SDK_DEPLOYMENT_ID_NOT_FOUND_ERROR


class SignalDeviceDomainNotFoundError(Exception):
    def __str__(self):
        return SDK_DEVICE_DOMAIN_NOT_FOUND_ERROR


class SignalAppParamNotFoundError(Exception):
    def __str__(self):
        return SDK_APP_PARAM_NOT_FOUND_ERROR


class SignalAppCallBackError(Exception):
    def __str__(self):
        return SDK_APP_CALL_BACK_ERROR


class SignalAppIoTConError(Exception):
    def __str__(self):
        return SDK_APP_IOT_CONN_ERROR


class SignalAppSignedReqError(Exception):
    def __str__(self):
        return SDK_APP_SIGNED_REQ_ERROR



import logging


def throw_if_parameter_not_found_in(value, param, location):
    """validate parameter existence in location
    """
    if not value:
        error_message = f"signalsdk::Application SDK can not start " \
                        f"because {param} is not found in {location}"
        logging.info(error_message)
        raise Exception(error_message)



import logging

import boto3
from botocore.auth import SigV4Auth
from botocore.awsrequest import AWSRequest
import requests
from signalsdk.signal_exception import SignalAppSignedReqError

def signed_request(method, url, data=None, params=None, headers=None):
    session = boto3.Session()
    credentials = session.get_credentials()
    creds = credentials.get_frozen_credentials()
    logging.info(f"signalsdk::accesskey: {creds.access_key}, secretkey: {creds.secret_key}")
    request = AWSRequest(method=method, url=url, data=data, params=params, headers=headers)
    SigV4Auth(creds, "execute-api", 'us-east-1').add_auth(request)
    response = requests.request(method=method,
                                url=url,
                                headers=dict(request.headers),
                                data=data,
                                timeout=300)
    if response.status_code == 200:
        return response.json()
    raise SignalAppSignedReqError



import logging
from typing import Callable
from paho.mqtt.client import Client, MQTT_ERR_SUCCESS

OnEventReceived=Callable[[str], None]

LOCAL_URL = "localhost"

class LocalMqtt():
    def __init__(self):
        self.mqttClient = Client("localmqtt")
        self.eventCallback = None
        self.subscribedTopic = None

    def set_on_event_received(self, callback):
        self.eventCallback = callback

    def on_message_received(self, client, userdata, message):
        payload = str(message.payload.decode("utf-8"))
        topic = message.topic
        logging.info(f"{__name__}: signalsdk: message received topic:{topic} payload:{payload}")
        try:
            self.eventCallback(payload)
        except TypeError:
            logging.info(f"{__name__}: signalsdk: on_message_received callback error")

    def on_disconnect(self, client, userdata, rc):
        logging.info(f"signalsdk::disconnecting reason: {str(rc)}")

    def on_connect(self, client, userdata, flags, rc):
        logging.info("signalsdk::Local MQTT connection established")

    def get_client(self):
        return self.mqttClient

    def connect(self):
        logging.info(f"signalsdk::{__name__}: signalsdk: connect...")

        self.mqttClient.on_message=self.on_message_received
        self.mqttClient.on_disconnect = self.on_disconnect
        self.mqttClient.on_connect = self.on_connect
        self.mqttClient.connect(LOCAL_URL)
        self.mqttClient.loop_start()

    def publish(self, topic, payload):
        try:
            self.mqttClient.publish(topic, payload)
        except Exception as e:
            logging.error(f"signalsdk::{e}: unable to publish on event bus...")

    def subscribe(self, topic):
        if topic and topic != self.subscribedTopic:
            result, _ = self.mqttClient.subscribe(topic)
            if result == MQTT_ERR_SUCCESS:
                logging.info(f"signalsdk::localmqtt subscribed topic: {topic}")
                self.subscribedTopic = topic
            else:
                logging.error(f"signalsdk::localmqtt Failed to subscribe topic: {topic}")

    def unsubscribe(self):
        logging.info(f"signalsdk::unsubscribing, topic:{self.subscribedTopic}")
        result, _ = self.mqttClient.unsubscribe(self.subscribedTopic)
        if result != MQTT_ERR_SUCCESS:
            print(f"signalsdk::localmqtt Failed to unsubscribe topic: {self.subscribedTopic}")
        self.subscribedTopic = None

    def get_subscribed_topic(self):
        return self.subscribedTopic



#!/usr/bin/env python

# Copyright 2014 OpenMarket Ltd
# Copyright 2017 Vector Creations Ltd
# Copyright 2019 The Matrix.org Foundation C.I.C.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from os import path
from setuptools import setup

__version__ = '0.1.8'

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as file:
    readme_file = file.read()

DEPENDENCIES = [
    "awsiotsdk>=1.7.1",
    "requests>=2.26.0",
    "awscrt==0.13.9",
    "awsiotsdk==1.11.0",
    "boto3==1.21.43",
    "botocore==1.24.43",
    "paho-mqtt==1.6.1",
    "urllib3==1.26.8",
    "python-dateutil==2.8.2"
]

TEST_DEPENDENCIES = [
    "pretend==1.0.9",
    "pytest==7.1.2",
    "pylint==2.13.9"
]
setup(
    name="signal-application-python-sdk",
    version=__version__,
    packages=['signalsdk'],
    description="signal application services",
    long_description=readme_file,
    long_description_content_type='text/markdown',
    install_requires=DEPENDENCIES,
    test_require=TEST_DEPENDENCIES,
    author='Wesley Clover',
    author_email='burak.cakmak@wesleyclover.com'
)
